classdef MF_BS_tool_inter_db < MF_BS_tool_inter
    %===========================================================================
    properties (Access = public)
        ndata = [];
        njmax = [];
        backup_interval = 0;
        backup_filename = 'backup_db.mat';
        backup_cmd_fun = @(filename) sprintf ('save (''%s'', ''-v7.3'');', filename);
        % function that builds the command to save backups, given a filename
        mean_fun = @mean;
    end  % public properties
    %===========================================================================
    methods (Access = public)
        %-----------------------------------------------------------------------
        function obj = MF_BS_tool_inter_db ()
            % Have to call superclass' constructor explicitly.
            % It already sets the path.
            obj = obj@MF_BS_tool_inter ();

            % Platform-indepent managing of path:
            path_str = fileparts (mfilename ('fullpath'));
            addpath (fullfile (path_str, 'misc', 'db'));

        end  % function MF_BS_tool_inter
        %-----------------------------------------------------------------------
        function analyze (self, data_fun)
            self.pack_structs ();
            self.check_params ();

            if nargin < 2
                % Reuse internal logstat
                % TODO: should check compatibility of parameters (e.g that p or
                % momnul haven't changed)
                [self.est, self.logstat, self.stat, self.mrq] = ...
                    MF_BS_tool (self.logstat, self.param_est, self.method_mrq, ...
                                self.param_bs, self.paramTEST, ...
                                self.param_display);
            else
                [self.est, self.logstat, self.stat] = ...
                    MF_BS_tool_db (data_fun, self.param_db, ...
                                   self.param_est, self.method_mrq, ...
                                   self.param_bs, self.paramTEST, ...
                                   self.param_display);
            end
        end  % function analyze
        %-----------------------------------------------------------------------
        function restore_from_backup (self, backup_filename)
            backup = load (backup_filename, ...
                           'param_est', 'param_bs', 'param_test', 'param_db', ...
                           'param_display', 'method_mrq');
            self.param_est     = backup.param_est;
            self.param_bs      = backup.param_bs;
            self.paramTEST     = backup.param_test;
            self.param_db      = backup.param_db;
            self.param_display = backup.param_display;
            self.method_mrq    = backup.method_mrq;

            [self.est, self.logstat, self.stat] = ...
                MF_BS_tool_db (backup_filename, self.param_db, ...
                               self.param_est, self.method_mrq, ...
                               self.param_bs, self.paramTEST, ...
                               self.param_display);

        end  % function restore_from_backup
        %-----------------------------------------------------------------------
    end  % public methods
    %===========================================================================
    properties (Access = protected)
        param_db;
    end  % private properties
    %===========================================================================
    methods (Access = protected)
        %-----------------------------------------------------------------------
        function pack_structs (self)
            pack_structs@MF_BS_tool_inter (self);

            self.param_db.ndata = self.ndata;
            self.param_db.njmax = self.njmax;
            self.param_db.backup_interval = self.backup_interval;
            self.param_db.backup_filename = self.backup_filename;
            self.param_db.backup_cmd_fun  = self.backup_cmd_fun;
            self.param_db.mean_fun = self.mean_fun;
        end % function pack_structs
        %-----------------------------------------------------------------------
        function check_params (self)
            check_params@MF_BS_tool_inter (self);

            % TODO:  checks for own params?

        end  % function check_params
        %-----------------------------------------------------------------------
    end  % private methods
    %===========================================================================
end % class MF_BS_tool_inter_db