function [wcoh] = wav_coherence1d (data, param_est)

% Compute pairwise wavelet coeherence of data.
%
% The input can be either a cell array with the 'coef' produced by DxPx1d, or
% a matrix of size N x K with K realizations of length N.

if iscell (data)
    coef = data;
    K = length (coef);
else      % Compute wavelet coefficients
    K = size (data, 2);
    for k = 1 : K
        [coef{k}] = DxPx1d (data(:, k), param_est);
    end
end

J = length (coef{1}.value);

% Coefficients might not have the same lengths due to the exclusion of NaNs
% during computations.
% Restore the NaNs using xpos, and then exclude them using nanmean.
for j = 1 : J
    % Get largest length of coefficients
    for k = 1 : K
        len(k) = coef{k}.xpos{j}(end);
    end
    lenmax(j) = max (len);

    Smat{j} = NaN (lenmax(j), k);
    for k = 1 : K
        Smat{j}(coef{k}.xpos{j}, k) = ...
            coef{k}.value{j} - nanmean (coef{k}.value{j});
    end
end

% Compute coherence of all pairs
wcoh = NaN (J, K, K);
for j = J : -1 : 1
    for k1 = K : -1 : 1
        for k2 = K : -1 : k1+1
            wcoh(j, k1, k2-1) = ...
                nanmean (Smat{j}(:, k1) .* conj (Smat{j}(:, k2))) ./ ...
                (sqrt (nanmean (Smat{j}(:, k1).^2) * nanmean (Smat{j}(:, k2).^2)) + eps);
        end
    end
end
