function [wcoh] = wav_coherence2d (data, param_est)

% Compute pairwise wavelet coeherence of 2d data.
%
% The input can be either a cell array with the 'coef' produced by DxPx2d, or
% a matrix of size N1 x N2 x K with K realizations of size N1 x N2.

if iscell (data)
    coef = data;
    K = length (coef);
else      % Compute wavelet coefficients
    K = size (data, 2);
    for k = 1 : K
        [coef{k}] = DxPx2d (data(:, :, k), param_est);
    end
end

J = length (coef{1}.value);

% Wavelet coefficients have 3 directions but leaders don't.
ndir = size(coef{1}.value{1}, 3);

% Coefficients might not have the same lengths due to the exclusion of NaNs
% during computations.
% Restore the NaNs using xpos and ypos, and then exclude them using nanmean.
for j = 1 : J
    % Get largest length of coefficients
    for k = 1 : K
        lenx(k) = coef{k}.xpos{j}(end);
        leny(k) = coef{k}.ypos{j}(end);
    end
    lenmaxx(j) = max (lenx);
    lenmaxy(j) = max (leny);

    Smat{j} = NaN (lenmaxx(j), lenmaxy(j), ndir, k);
    sz = size (coef{k}.value{j});
    for k = 1 : K
        means = nanmean (nanmean (coef{k}.value{j}, 1), 2);
        Smat{j}(coef{k}.xpos{j}, coef{k}.ypos{j}, :, k) = ...
            coef{k}.value{j} - repmat (means, sz(1), sz(2), 1);  % subtraction broadcasted
    end
end

% Compute coherence of all pairs
wcoh = NaN (J, K, K);
for j = J : -1 : 1
    for k1 = K : -1 : 1
        for k2 = K : -1 : k1+1
            a = Smat{j}(:, :, :, k1);
            b = Smat{j}(:, :, :, k2);
            prod = Smat{j}(:, :, :, k1) .* conj (Smat{j}(:, :, :, k2));
            wcoh(j, k1, k2-1) = ...
                nanmean (prod(:)) ./ ...
                (sqrt (nanmean (a(:).^2) * nanmean (b(:).^2)) + eps);
        end
    end
end
