%- Basic test for Grand Canonical toolbox
%
% 2017-08-30

% Add to path the root folder of the toolbox, where MF_BS_tool_inter is located.
% The toolbox takes care of the rest.
addpath ..
addpath WLBMF_example_data

N = 2^12;

%% CREATE DATA EXAMPLES

%
% The components must be stored in a cell array.
% So for now only 2 components are supported.
%

%close all

EXAMPLE_SELECT = 5;

bm  = cumsum (randn (N, 1));
bm2 = cumsum (randn (N, 1));

switch EXAMPLE_SELECT
  case 1
    data = {bm, bm2};
  case 2
    data = {bm, bm};
  case 3  % check old code
    data = bm;
  case 4
    H = 0.7;
    lambda = sqrt (0.04);
    m1 = cumsum (mrw (H, lambda, N, N, 1));
    m2 = cumsum (mrw (H, lambda, N, N, 1));

    data = {m1, m2};
  case 5
    H = 0.7;
    lambda = sqrt (0.04);
    m1 = cumsum (mrw (H, lambda, N, N, 1));

    data = {m1, m1};
  case 6
    rho = 0.4;
    S = mvnrnd([0;0], [[1 rho];[rho 1]], N);
    data = {S(:,1), S(:, 2)};
end


%% ANALYSIS

%
% The interface for the toolbox is exactly the same than before.
%
% The only new parameter is mo.q2.
% If mo.q2 is not provided, mo.q2 = mo.q1 is used.
%
% The analysis function detects when it receives a cell array, and
% and enables the grand canonical formalism.
%
% If a single data array is used, the old version of the toolbox should be used.
%

mo = MF_BS_tool_inter ();
mo.p = inf;
mo.q = -5 : 5;
%mo.q2 = -3 : 3;
mo.cum = 2;
mo.j1 = 4;
mo.j2 = 8;
mo.verbosity = 1;
mo.fig_num = 100;

mo.analyze (data);

%% STRAIGHTFORWARD WAY TO GET GRAND CANONICAL ESTIMATES

%
% The function reshape_estimates_GC receives 'est' or 'logstat' and returns
% structures containing matrices with all the estimates:
%

lgt_mat = reshape_estimates_GC (mo.logstat);
est_mat = reshape_estimates_GC (mo.est);

%
% ----- Logscale diagrams for all quantities ----
% lgt_mat.LWT.zq (nq1 x nq2 x nj)
% lgt_mat.LWT.Dq (nq1 x nq2 x nj)
% lgt_mat.LWT.hq (nq1 x nq2 x 2 x nj)
%    Third dimension encodes h1(q1,q2) and h2(q1, q2)
% lgt_mat.LWT.cp (ncum x nj)
%    Cumulants in 1st dim are encoded by order, as:
%       [c10, c01, c20, c11, c02, c30, c21, c12, c03, ...]
%
%
% ----- Estimates (slopes) for all quantities -----
% est_mat.LWT.t.zq (nq1 x nq2)
% est_mat.LWT.t.Dq (nq1 x nq2)
% est_mat.LWT.t.hq (nq1 x nq2 x 2)
% est_mat.LWT.t.cp (ncum)
%
%
% ----- Estimates (interecepts) for all quantities -----
% est_mat.LWT.aest.zq (nq1 x nq2)
% est_mat.LWT.aest.Dq (nq1 x nq2)
% est_mat.LWT.aest.hq (nq1 x nq2 x 2)
% est_mat.LWT.aest.cp (ncum)
%
% (I had to add this because I needed it for the plots)
