% This demo illustrates how to perform the multifractal analysis of an entire
% database, stored in a directory.
%
% Roberto Fabio Leonarduzzi
% 2016-09-06

clear all

%% Create example data
addpath (fullfile (fileparts (mfilename ('fullpath')), 'synth'))

eg_data_folder = fullfile (fileparts (mfilename ('fullpath')), 'eg');
mkdir (eg_data_folder);

% Create realizations of MRW:
ndata = 10;
N = 2^16;
c1 = 0.8;
c2 = -0.08;
H = c1 + c2 / 2;
lambda = sqrt (-c2);

fprintf ('\nCreating example data...\n\n')

for k = 1 : ndata
    data = mrw (H, lambda, N + 1, N + 1, 1);
    data = cumsum (data (2 : end));
    save (fullfile (eg_data_folder, sprintf ('eg_data_%02i.mat', k)), 'data');
end

%% Toolbox setup

mf_obj = MF_BS_tool_inter_db;

mf_obj.j1 = 3;
mf_obj.j2 = 10;
mf_obj.verbosity = 11;
mf_obj.fig_num = 100;
mf_obj.TEST = 1;
mf_obj.CI = 1;

mf_obj.mean_fun = @nanmean;

% Select way to present results:
mf_obj.bs_plot_type = 'superimpose';

%% Analysis: data from directory

% Select function to read  database:
reader = create_db_reader_dir (eg_data_folder);

fprintf ('\nStarting analysis...\n\n')
mf_obj.analyze (reader);

%% Remove example data

fprintf ('\nRemoving example data...\n\n')
rmdir (eg_data_folder, 's');
