% This demos illustrates how to perform a slinding-window multifractal
% analysis.
%
% Roberto leonarduzzi
% 2016-09-06

clear all

% Add synthesis functions to path safely
addpath (fullfile (fileparts (mfilename ('fullpath')), 'synth'));


%% Example data synthesis
% Synthesize a realization of MRW as an expample.

% MRW parameters
N = 2^16;
c1 = 0.8;
c2 = -0.08;
H = c1 + c2 / 2;
lambda = sqrt (-c2);

% Synthesis function
data = cumsum (mrw (H, lambda, N, N, 1));

%% Toolbox setup

mf_obj = MF_BS_tool_inter_db;

mf_obj.j1 = 3;
mf_obj.j2 = 10;
mf_obj.verbosity = 11;
mf_obj.fig_num = 100;
mf_obj.TEST = 1;
mf_obj.CI = 1;

mf_obj.mean_fun = @nanmean;

% Select way to present results:
mf_obj.bs_plot_type = 'superimpose';

%% Sliding window setup

winlen = 2^14;
noverlap = 0;

% Select function to do Monte Carlo analysis
reader = create_db_reader_slidewin (data, winlen, noverlap);

%% Analysis

mf_obj.analyze (reader);
