function [reader] = create_db_reader_dir (dirname, readfun, verbosity)
%function [reader] = create_db_reader_dir (dirname, readfun)
%
% Reads sequentially the files in folder dirname.
% dirname can contain the file matching patterns accepted by the dir function.
%  Reads each file using function readfun, which defaults to load.
%
% Returns a structure with fields:
%    - fun: closure that reads a new file in each call
%    - ndata: amount of data that will be read
%
% Roberto Leonarduzzi
% 2016-07-05

    if nargin < 2 || isempty (readfun)
        readfun = @load_wrapper;
    end
    if nargin < 3 || isempty (verbosity)
        verbosity = false;
    end

    % Read list of file names:
    list = dir (dirname);

    % Remove folders that might have been included
    ifile = 1;
    while ifile <= length (list)
        if list(ifile).isdir
            list(ifile) = [];
        else
            ifile = ifile + 1;
        end
    end

    reader.ndata = length (list);

    % Initialize file counter:
    ifile = 1;

    function [data] = fun_tmp ()
        if ifile > length (list)
            data = [];
            return
        end

        if verbosity
            fprintf ('Reading %s\n', fullfile (dirname, list(ifile).name));
        end

        data = readfun (fullfile (dirname, list(ifile).name));
        ifile = ifile + 1;
    end

    reader.fun = @fun_tmp;
end
