function [reader] = create_db_reader_filelist (filename, readfun)
%function [reader] = create_db_reader_filelist (filename, readfun)
%
% Read sequentially the files indicated in each line of file 'filename'.
% Reads each file using function readfun, which defaults to load.
%
% Returns a structure with fields:
%    - fun: closure that reads a new file in each call
%    - ndata: number of files that will be read.
%
% Roberto Leonarduzzi
% 2016-07-05

    if nargin < 2
        readfun = @load_wrapper;
    end

    % Read list of file names:
    fid = fopen (filename);
    if fid < 3
        error ('Could not open file %s.', filename)
    end
    list = textscan (fid, '%s');
    list = list{1};
    fclose (fid);

    reader.ndata = length (list);

    % Initialize file counter:
    ifile = 1;

    function [data] = fun_tmp ()
        if ifile > length (list)
            data = [];
            return
        end
        data = readfun (list{ifile});
        ifile = ifile + 1;
    end

    reader.fun = @fun_tmp;
end
