function [reader] = create_db_reader_mat (mat, dim)
% Output function that reads sequentially the data data stored in mat, along
% dimension dim.
%
% Roberto Leonarduzzi
% 2016-07-05

    if nargin < 2
        dim = 1;
    end

    n = size (mat, dim);
    counter = 1;

    % I will use a comma-separated list to handle the desired dimension. See
    % http://blogs.mathworks.com/loren/2006/03/22/making-functions-suitable-for-nd-arrays/

    % First, create a cell array of : to access all dimensions
    idx = repmat ({':'}, 1, max (ndims (mat), dim));

    function [data] = fun_tmp ()
        if counter > n
            data = [];
            return
        end

        % Index the desired dimension with the counter:
        idx{dim} = counter;

        % Then index the matrix using the comma-separated list generated from
        % the cell array:
        data = squeeze (mat(idx{:}));

        counter = counter + 1;
    end

    reader.ndata = n;
    reader.fun = @fun_tmp;
end
