function [idz, idd, idh, idc] = get_estimate_id (param_est)
% Gets the id for each type of estimate, according to param_est.estimate_select.
%
%  - param_est.estimate_select: number xyz - what is to be calculated:
%    * x : zeta(q) [0 or 1]    (scaling exponents)
%    * y : D(h) [0 or 1]       (multifractal spectrum)
%    * z : cp [0 or 1]         (log-cumulants)
%
% February, 2016

idz = [];
idd = [];
idh = [];
idc = [];

[flag_zq, flag_dh, flag_cp] = which_estimates (param_est.estimate_select);
nq1 = length (param_est.q);
nq2 = length (param_est.q2);
ncum = param_est.cum;

if nq2 == 0
    nq = nq1;
    nh = 1;
else
    nq = nq1 * nq2;
    ncum = 2 * ncum;
    % FIXME: add in a case by case basis the cross-cumulants
    % Fix with general solution once cumulants of order up to 5 are computed
    if param_est.cum > 1
        ncum = ncum + 1;  % c_{1,1}
    end
    if param_est.cum > 2
        ncum = ncum + 2;  % c_{2,1} and c_{1,2}
    end
    nh = 2;
end

id_q = 1 : nq;
id_c = 1 : ncum;

if flag_zq
    idz = 1 : nq;
end

if flag_dh
    idd = (1 : nq) + flag_zq * nq;
    idh = (1 : (nh * nq)) + (flag_zq + 1) * nq;
end

if flag_cp
    idc = (1 : ncum) + (flag_zq + (1 + nh) * flag_dh) * nq;
end
