function plot_logscale (ax, plot_data, param_est, param_bs)
%
%   - plot_data: structure with stuff to plot and parameters. Fields
%      * yy: "structure function"
%      * yy_bs: bootstrap replicates of "structure functions"
%      * slope, inter: slope and intercept of estimated linear model
%      * ylabel, title

if ~isfield (plot_data, 'fontsize_small')
    fontsize_small = 8;
else
    fontsize_small = plot_data.fontsize_small;
end

if ~isfield (plot_data, 'fontsize_large')
    fontsize_large = 12;
else
    fontsize_large = plot_data.fontsize_large;
end

if ~isfield (plot_data, 'linespec_estimates')
    linespec_estimates = 'k.-';
else
    linespec_estimates = plot_data.linespec_estimates;
end

if ~isfield (plot_data, 'linespec_regression_all')
    linespec_regression_all = 'b--';
else
    linespec_regression_all = plot_data.linespec_regression_all;
end

if ~isfield (plot_data, 'linespec_regression_emph')
    linespec_regression_emph = 'b-';
else
    linespec_regression_emph = plot_data.linespec_regression_emph;
end

if ~isfield (plot_data, 'flag_decorate')
    flag_decorate = true;
else
    flag_decorate = plot_data.flag_decorate;
end

if isequal (plot_data.bs_plot_logscale_fun, @plot_logscale_superimposed)
    lw = 2;
else
    lw = 0.5;
end

flag_plot_regression = ~(isempty (plot_data.slope) || isempty (plot_data.inter));

jj = 1 : length (plot_data.yy);  % x data
yy = plot_data.yy;  % y data

scal_rng = param_est.j1 : param_est.j2;   % Indices of scales used for bootstrap
if flag_plot_regression
    linear_est = plot_data.slope * jj + plot_data.inter;
end

axes (ax);
hold on
%-- 1: plot confidence intervals if appropriate
plot_data.bs_plot_logscale_fun (ax, plot_data, param_est, param_bs);

%-- 2: plot estimates
plot (jj, yy, linespec_estimates, 'LineWidth', lw)

%-- 3:  plot linear estimate
if flag_plot_regression
    % Plot in full range
    plot (jj, linear_est, linespec_regression_all)

    % Emphasize linear estimate in used scaling range
    plot (jj(scal_rng), linear_est(scal_rng), linespec_regression_emph)
end
hold off
grid on

if flag_decorate
    if flag_plot_regression && plot_data.slope < 0
        posX = 0.7;
    else
        posX = 0.03;
    end
    if flag_plot_regression
        text ('String', num2str(plot_data.slope), ...
              'FontSize', fontsize_large, ...
              'FontName', 'Helvetica', ...
              'Color', 'red', ...
              'Units','normalized', ...
              'Pos',[posX 0.9]);
    end
    text ('String', sprintf ('wtype %i', param_est.wtype), ...
          'FontSize', fontsize_small, ...
          'FontName', 'Helvetica', ...
          'Units', 'normalized', ...
          'Pos',[posX 0.76]);
    text ('String', sprintf ('j:[%i-%i]', param_est.j1, param_est.j2), ...
          'FontSize', fontsize_small, ...
          'FontName', 'Helvetica', ...
          'Units', 'normalized', ...
          'Pos',[posX 0.69]);
end

% Setup callback for plotting in separate figure if clicked:
children = get (ax, 'Children');
set (children, 'HitTest', 'off');
set (ax, 'ButtonDownFcn', 'my_click_and_plot (gcbo)')
