function plot_surf (plot_data, param_est, param_bs)
%   - plot_data: structure with stuff to plot and parameters. Fields
%      * hfig: handle of figure to use
%      * xx, yy: matrices of arguments (q, or h)
%      * zz: matriz of estimates (zeta or D)
%      * ylabel, title

if ~isfield (plot_data, 'fontsize_small')
    fontsize_small = 8;
else
    fontsize_small = plot_data.fontsize_small;
end

if ~isfield (plot_data, 'fontsize_large')
    fontsize_large = 12;
else
    fontsize_large = plot_data.fontsize_large;
end

if ~isfield (plot_data, 'plot_projections')
    plot_projections = false;
else
    plot_projections = plot_data.plot_projections;
end

% Unpack data
xx = plot_data.xx;  % x data
yy = plot_data.yy;  % x data
zz = plot_data.zz;  % y data
x_str = plot_data.x_str;
y_str = plot_data.y_str;
z_str = plot_data.z_str;

x_lim = [];
y_lim = [];
z_lim = [];

if isfield (plot_data, 'z_lim')
    z_lim = plot_data.z_lim;
    % Add margin and expand if needed
    z_lim(2) = z_lim(2) * 1.2;
    z_lim(2) = max (z_lim(2), max (max (zz)));
end
if isfield (plot_data, 'x_lim')
    x_lim = plot_data.x_lim;
    % Expand if needed
    x_lim(1) = min (x_lim(1), min (min (xx)));
    x_lim(2) = max (x_lim(2), max (max (xx)));
end
if isfield (plot_data, 'y_lim')
    y_lim = plot_data.y_lim;
    % Expand if needed
    y_lim(1) = min (y_lim(1), min (min (yy)));
    y_lim(2) = max (y_lim(2), max (max (yy)));
end

%========== plot surface ====================
ax = gca;

surf (xx, yy, zz);
if plot_projections
    hold on
    color = 0.4 * [1 1 1];
    mesh (xx * 0 + x_lim(end) , yy , zz , ...
          'EdgeColor' , color, 'FaceColor', 'none')
    mesh (xx , yy * 0 + y_lim(end) , zz , ...
          'EdgeColor' , color, 'FaceColor', 'none')
    mesh (xx , yy , zz * 0 , ...
          'EdgeColor' , color, 'FaceColor', 'none')
end
xlabel (x_str)
ylabel (y_str)
zlabel (sprintf ('%s(%s,%s)', z_str, x_str, y_str))
title ('Blue: marginal. Gray: projection')
if ~isempty (z_lim)
    zlim (z_lim)
end
if ~isempty (x_lim)
    xlim (x_lim)
end
if ~isempty (y_lim)
    ylim (y_lim)
end
%========== plot marginal: x, y=0 ====================
[~, ix0] = min (abs (param_est.q2));

z_mar = zz(:, ix0);
x_mar = xx(:, ix0);
[z_env, idx] = max (zz, [], 1);
for ii = 1 : length (idx)
    x_env(ii) = xx(idx(ii), ii);
end

if plot_projections
    plot3 (ax, x_mar, 0 * x_mar + y_lim(end), z_mar, 'b', 'LineWidth', 2)
end

%========== plot marginal: x=0, y ====================
[~, ix0] = min (abs (param_est.q));

z_mar = zz(ix0, :);
x_mar = yy(ix0, :);
[z_env, idx] = max (zz, [], 2);
for ii = 1 : length (idx)
    x_env(ii) = yy(ii, idx(ii));
end

if plot_projections
    ax;
    plot3 (ax, 0 * x_mar + x_lim(end),  x_mar, z_mar, 'b', 'LineWidth', 2)
end

% Syntax 'text (ax, ...)' dosn't work in old versions of matlab.
% Works in 2017a, doesn't work in 2014b, 2015b.
% Activate the desired axis instead.
axes (ax);
text ('String', sprintf ('Frac Int %g', param_est.gamint), ...
      'FontSize', fontsize_small, ...
      'FontName', 'Helvetica', ...
      'Units', 'normalized', ...
      'Pos', [0.03 0.93]);
text ('String', sprintf ('N_{\\psi} %i', param_est.nwt), ...
      'FontSize', fontsize_small, ...
      'FontName', 'Helvetica', ...
      'Units', 'normalized', ...
      'Pos', [0.03 0.86]);
text ('String', sprintf ('j:[%i-%i]', param_est.j1, param_est.j2), ...
      'FontSize', fontsize_small, ...
      'FontName', 'Helvetica', ...
      'Units', 'normalized', ...
      'Pos', [0.03 0.79]);
text ('String', sprintf ('wtype %i', param_est.wtype), ...
      'FontSize', fontsize_small, ...
      'FontName', 'Helvetica', ...
      'Units','normalized', ...
      'Pos', [0.03 0.72]);


% Setup callback for plotting in separate figure if clicked:
children = get (ax, 'Children');
set (children, 'HitTest', 'off');
set (ax, 'ButtonDownFcn', 'my_click_and_plot (gcbo)')
