function plot_xy (ax, plot_data, param_est, param_bs)
%
%   - plot_data: structure with stuff to plot and parameters. Fields
%      * yy: "structure function"
%      * yy_bs: bootstrap replicates of "structure functions"
%      * slope, inter: slope and intercept of estimated linear model
%      * ylabel, title

if ~isfield (plot_data, 'fontsize_small')
    fontsize_small = 8;
else
    fontsize_small = plot_data.fontsize_small;
end

if ~isfield (plot_data, 'fontsize_large')
    fontsize_large = 12;
else
    fontsize_large = plot_data.fontsize_large;
end

if ~isfield (plot_data, 'linespec_estimates')
    linespec_estimates = 'k.-';
else
    linespec_estimates = plot_data.linespec_estimates;
end

if ~isfield (plot_data, 'flag_decorate')
    flag_decorate = true;
else
    flag_decorate = plot_data.flag_decorate;
end

if isequal (plot_data.bs_plot_xy_fun, @plot_xy_superimposed)
    lw = 2;
else
    lw = 0.5;
end

% Unpack data
xx = plot_data.xx;  % x data
yy = plot_data.yy;  % y data

axes (ax);
hold on

%-- 1: plot confidence intervals if appropriate
plot_data.bs_plot_xy_fun (ax, plot_data, param_est, param_bs)

%-- 2: plot estimates
plot (xx, yy, linespec_estimates, 'LineWidth', lw)
hold off
grid on

if flag_decorate
    text ('String', sprintf ('Frac Int %g', param_est.gamint), ...
          'FontSize', fontsize_small, ...
          'FontName', 'Helvetica', ...
          'Units', 'normalized', ...
          'Pos', [0.03 0.93]);
    text ('String', sprintf ('N_{\\psi} %i', param_est.nwt), ...
          'FontSize', fontsize_small, ...
          'FontName', 'Helvetica', ...
          'Units', 'normalized', ...
          'Pos', [0.03 0.86]);
    text ('String', sprintf ('j:[%i-%i]', param_est.j1, param_est.j2), ...
          'FontSize', fontsize_small, ...
          'FontName', 'Helvetica', ...
          'Units', 'normalized', ...
          'Pos', [0.03 0.79]);
    text ('String', sprintf ('wtype %i', param_est.wtype), ...
          'FontSize', fontsize_small, ...
          'FontName', 'Helvetica', ...
          'Units','normalized', ...
          'Pos', [0.03 0.72]);
    text ('String', sprintf ('(p_0=%i)', round (plot_data.p0noint * 100) / 100), ...
          'FontSize', fontsize_large, ...
          'FontName', 'Helvetica', ...
          'Color', 'red', ...
          'Units', 'normalized', ...
          'pos', [0.5 0.2], ...
          'HorizontalAlignment', 'center');
    text ('String', sprintf ('p_0^{(\\gamma)}=%i', round (plot_data.p0 * 100) / 100), ...
          'FontSize', fontsize_large, ...
          'FontName', 'Helvetica', ...
          'Color', 'red', ...
          'Units', 'normalized', ...
          'Pos', [0.5 0.07], ...
          'HorizontalAlignment', 'center');
end

% Setup callback for plotting in separate figure if clicked:
children = get (ax, 'Children');
set (children, 'HitTest', 'off');
set (ax, 'ButtonDownFcn', 'my_click_and_plot (gcbo)')