function [lgt_mat] = reshape_estimates_GC (s)
% Reshapes the vectorized estimates to matrix form.
% Works for est and logstat.

% if the "root" est or logtat are provided, analyze recursively DWT and LWT
% and exit
% Otherwise proceed directly on s.
if isfield (s, 'DWT') || isfield (s, 'LWT')
    if isfield (s, 'DWT')
        [lgt_mat.DWT] = reshape_estimates_GC (s.DWT);
    end
    if isfield (s, 'LWT')
        [lgt_mat.LWT] = reshape_estimates_GC (s.LWT);
    end
    return
end

sz_est = s.sz_est;
param_est = s.param_est;

[flag_zq, flag_dh, flag_cp] = which_estimates (param_est.estimate_select);
[idz, idd, idh, idc] = get_estimate_id (param_est);

if isfield (s, 't')   % structure 'est'
    [ss, dd, hh, cc] = reshape_vector (s.t);
    lgt_mat.t.zq = ss;
    lgt_mat.t.Dq = dd;
    lgt_mat.t.hq = hh;
    lgt_mat.t.cp = cc;

    [ss, dd, hh, cc] = reshape_vector (s.aest);
    lgt_mat.aest.zq = ss;
    lgt_mat.aest.Dq = dd;
    lgt_mat.aest.hq = hh;
    lgt_mat.aest.cp = cc;
elseif isfield (s, 'est')   % structure 'logstat'
    nj = size (s.est, 2);
    for ij = nj : -1 : 1
        [ss, dd, hh, cc] = reshape_vector (s.est(:, ij));
        lgt_mat.zq(:, :, ij)    = ss;
        lgt_mat.Dq(:, :, ij)    = dd;
        lgt_mat.hq(:, :, :, ij) = hh;
        lgt_mat.cp(:, ij)    = cc;
    end
else
    error ('Unrecognized structure.')
end


function [Sq, Dq, hq, cp] = reshape_vector (vec)
    Sq = NaN;
    Dq = NaN;
    hq = NaN;
    cp = NaN;

    if flag_zq
        Sq = reshape (vec(idz), sz_est{1});
    end
    if flag_dh
        Dq = reshape (vec(idd), sz_est{2});
        hq = reshape (vec(idh), sz_est{3});
    end
    if flag_cp
        cp = reshape (vec(idc), sz_est{4});
    end

    % squeeze to be elegant in non-GC case:
    Sq = squeeze (Sq);
    Dq = squeeze (Dq);
    hq = squeeze (hq);
    cp = squeeze (cp);
end  % end function reshape_vector

end   % function reshape estimates
