%- Test for DxPx2d
% 
% Test the values of logstat and est that were computed with a canonical version
% and stored.
%
% RFL, January 2016

clear variables

%===============================================================================
% PATH TO VERSION THAT WILL BE TESTED
%====================================
addpath /Users/roberto/Dropbox/postdoc/PLBMF_tool_singul_git
pathMF_BS_tool_singul

%===============================================================================
% Test for p-exponent, p = \infty

% Load reference data
reference = load ('test_cpc2d_dx_px_p0.mat');

[coef, leader] = DxPx2d (reference.data, reference.param_est);

% Make tests for coefficients:
result = true;
clear tf
for j = 1 : length (coef.value)
    % Make equivalent concatenation:
    samplex  = reshape (coef.value{j}(:, :, 1), 1, []);
    sampley  = reshape (coef.value{j}(:, :, 2), 1, []);
    samplexy = reshape (coef.value{j}(:, :, 3), 1, []);
    sample   = abs ([samplex sampley samplexy]);
    tf(j) = all (sample == reference.coef(j).vector.all);
end
result = result & all (tf);

if result
    fprintf ('Wavelet coefficients: no differences were found\n')
else
    fprintf ('Wavelet coefficients: FOUND DIFFERENCES!!!!\n')
    tf
end


% Make tests for leaders:
result = true;
clear tf
for j = 1 : length (coef.value)
    sample = reshape (leader.value{j}(:), 1, []);
    tf(j) = all (sample == reference.leader(j).vector.max);
end
result = result & all (tf);

if result
    fprintf ('Wavelet leaders: no differences were found\n')
else
    fprintf ('Wavelet leaders: FOUND DIFFERENCES!!!!\n')
    tf
end
%===============================================================================

%===============================================================================
% Test for p-exponent, p = 2, without correction

% Load reference data
reference = load ('test_cpc2d_dx_px_p2_nocorr.mat');

[coef, leader] = DxPx2d (reference.data, reference.param_est);

% Make tests for leaders:
result = true;
clear tf
for j = 1 : length (coef.value)
    sample = reshape (leader.value{j}(:), 1, []);
    tf(j) = all (sample == reference.leader(j).vector.sum);
end
result = result & all (tf);

if result
    fprintf ('2-leaders, no correction: no differences were found\n')
else
    fprintf ('2-leaders, no correction: FOUND DIFFERENCES!!!!\n')
    tf
end
%===============================================================================

%===============================================================================
% Test for p-exponent, p = 2, with correction

% Load reference data
reference = load ('test_cpc2d_dx_px_p2.mat');

[coef, leader] = DxPx2d (reference.data, reference.param_est);

% Make tests for leaders:
result = true;
clear tf
for j = 1 : length (coef.value)
    sample = reshape (leader.value{j}(:), 1, []);
    tf(j) = all (sample == reference.leader(j).vector.sum);
end
result = result & all (tf);

if result
    fprintf ('2-leaders: no differences were found\n')
else
    fprintf ('2-leaders: FOUND DIFFERENCES!!!!\n')
    tf
end
%===============================================================================