% Scrpt to test the backup features of MF_BS_tool_db
%
% Make a full analysis of a matrix, and then a second analysis that will be
% stopped. Recover from the backup, continue analysis and compare results
%
% Roberto Leonarduzzi, 2017-03-09

clear all
clear functions

% Add toolbox to path
addpath ..

%% Create example data
old_path = addpath (fullfile (fileparts (mfilename ('fullpath')), 'synth'));


% Create realizations of MRW:
ndata = 10;
N = 2^16;
c1 = 0.8;
c2 = -0.08;
H = c1 + c2 / 2;
lambda = sqrt (-c2);

fprintf ('\nCreating example data...\n\n')

for k = 1 : ndata
    data = mrw (H, lambda, N + 1, N + 1, 1);
    data = cumsum (data (2 : end));
    data_matrix(:, k) = data;
end

%% Toolbox setup

mf_obj = MF_BS_tool_inter_db;

mf_obj.j1 = 3;
mf_obj.j2 = 10;
mf_obj.verbosity = 1;
mf_obj.fig_num = 100;
mf_obj.TEST = 1;
mf_obj.CI = 1;
mf_obj.mean_fun = @nanmean;

% Select way to present results:
mf_obj.bs_plot_type = 'superimpose';



%% Analysis full

mf_obj_full = copy (mf_obj);

reader = create_db_reader_mat (data_matrix, 2);

mf_obj_full.analyze (reader);

%% Analysis interrupted, recovery from backup

try
    rmdir ('flag_dont_inerrupt')
end

reader = create_db_reader_mat_interrupt (data_matrix, 2, 5);

mf_obj_rec = copy (mf_obj);

mf_obj_rec.fig_num = 200;
mf_obj_rec.backup_filename = 'backup_db.mat';
mf_obj_rec.backup_interval = 2;


try
    mf_obj_rec.analyze (reader);
catch ME
    fprintf ('%s\n', ME.message)
end
fprintf ('Recovering from backup...\n')

mf_obj_rec.restore_from_backup (mf_obj_rec.backup_filename);

rmdir ('flag_dont_inerrupt')

fprintf ('\n')
if isequal (mf_obj_full.logstat, mf_obj_rec.logstat);
    fprintf ('logstat ARE equal\n')
else
    fprintf ('logstat ARE NOT equal\n')
end

if isequal (mf_obj_full.est, mf_obj_rec.est)
    fprintf ('est ARE equal\n')
else
    fprintf ('est ARE NOT equal\n')
end
